<?php
App::uses('ErrorHandler', 'Error');
use Airbrake\Notifier;
use Airbrake\Instance;


class Errbit extends ErrorHandler{

    private static $errbit;
	private static $notifier;

	/**
	 * Init the airbrake/errbit api instance (intentionally not in the constructor)
	 *
	 * @return void
	 */
	public static function init() {

		if (
			empty(Configure::read('EvErrbit.projectKey')) ||
			empty(Configure::read('EvErrbit.projectId'))) {
			// If the project key or project id are missing, then the plugin
			// has not been configured so we dont want to use it.
			return false;
		}

		self::$notifier = new Airbrake\Notifier([
			'projectId' => Configure::read('EvErrbit.projectId'),
			'projectKey' => Configure::read('EvErrbit.projectKey'),
			'host' => Configure::read('EvErrbit.host'),
			'rootDirectory' => '/',
			'httpClient' => 'curl',
			'appVersion' => '1'

		]);

		Airbrake\Instance::set(self::$notifier);

		self::$errbit = new Airbrake\ErrorHandler(self::$notifier);
		self::$errbit->register();
	}

	/**
	 * Catch errors and run them through Errbit before passing back to cake
	 */
    public static function handleError($code, $description, $file = null, $line = null, $context = null) {
		if (self::init()) {
			self::onError($code, $description, $file, $line);
		}

		return parent::handleError($code, $description, $file, $line, $context);
    }

	/**
	 * Catch exceptions and run them through Errbit before passing back to cake
	 */
    public static function handleException(Exception $exception, $reportOnly = false) {
		if (self::init()) {
	        Airbrake\Instance::notify($exception);
		}

		if ($reportOnly) {
			return true;
		}

        return parent::handleException($exception);
    }

	/**
	 * Handles different error types, pasing them to Airbrake/Errbit's different
	 * handlers
	 */
	public static function onError($code, $message, $file, $line)
    {
        switch ($code) {
            case E_NOTICE:
            case E_USER_NOTICE:
                $exc = new Airbrake\Errors\Notice($message, debug_backtrace());
                break;
            case E_WARNING:
            case E_USER_WARNING:
                $exc = new Airbrake\Errors\Warning($message, debug_backtrace());
                break;
            case E_ERROR:
            case E_CORE_ERROR:
            case E_RECOVERABLE_ERROR:
                $exc = new Airbrake\Errors\Fatal($message, debug_backtrace());
                break;
            case E_USER_ERROR:
            default:
                $exc = new Airbrake\Errors\Error($message, debug_backtrace());
        }
        self::$notifier->notify($exc);
    }
}
